@@ -52,8 +52,9 @@ def filter_pkglist(pkglist, options):
 	return result
 
 def display_pkg(pkg, options):
+	displaystyle = 'file' if options.package else 'local'
 	if options.info > 0:
-		pkginfo.display_pkginfo(pkg, level = options.info, style = 'local')
+		pkginfo.display_pkginfo(pkg, level = options.info, style = displaystyle)
 	elif not options.listfiles:
 		if options.quiet:
 			print(pkg.name)
@@ -117,6 +118,19 @@ def find_file(filenames, options):
 
 	return ret
 
+def find_search(patterns, options):
+	db = pyalpm.get_localdb()
+	results = db.search(*patterns)
+	if len(results) == 0:
+		return 1
+	for pkg in results:
+		if options.quiet:
+			print(pkg.name)
+		else:
+			print("%s/%s %s" % (pkg.db.name, pkg.name, pkg.version))
+			print("    " + pkg.desc)
+	return 0
+
 def main(rawargs):
 	parser = config.make_parser(prog = 'pycman-query')
 	group = parser.add_argument_group("Query options")
@@ -138,9 +152,14 @@ def main(rawargs):
 	group.add_argument('-o', '--owns',
 			action = 'store_true', default = False,
 			help = 'query the package that owns <file>')
+	group.add_argument('-p', '--package',
+			action = 'store_true', default = False,
+			help = 'query a package file instead of the database')
 	group.add_argument('-q', '--quiet',
 			action = 'store_true', dest = 'quiet', default = False,
 			help = 'show less information for query and search')
+	group.add_argument('-s', '--search', action = 'store_true', default = False,
+			help = 'search locally-installed packages for matching strings')
 	group.add_argument('-t', '--unrequired',
 			action = 'store_true', default = False,
 			help = "list packages not required by any package [filter]")
@@ -153,7 +172,7 @@ def main(rawargs):
 			'when used with -p: the path to a package file)')
 
 	args = parser.parse_args(rawargs)
-	config.init_with_config(args)
+	config.init_with_config_and_options(args)
 
 	if args.verbose:
 		print("query " + " ".join(rawargs), file = sys.stderr)
@@ -164,12 +183,17 @@ def main(rawargs):
 	# actions other than listing packages
 	if args.owns:
 		return find_file(args.pkgnames, args)
+	if args.search:
+		return find_search(args.pkgnames, args)
 
 	pkglist = []
 	if len(args.pkgnames) > 0:
 		# a list of package names was specified
 		for pkgname in args.pkgnames:
-			pkg = db.get_pkg(pkgname)
+			if args.package:
+				pkg = pyalpm.load_pkg(pkgname)
+			else:
+				pkg = db.get_pkg(pkgname)
 			if pkg is None:
 				print('error: package "%s" not found' % pkgname)
 				retcode = 1
