@@ -25,18 +25,38 @@ This module handles pacman.conf files as well as pycman options that
 are common to all action modes.
 """
 
+import io
+import os
+import sys
 import argparse
 import configparser
 
 import pyalpm
 
 def read_config(path):
+	f = open(path)
+	preprocessed = io.StringIO()
+	for line in f:
+		if '=' in line:
+			# Process Include statements
+			key, value = line.split('=', 1)
+			key = key.strip()
+			value = value.strip()
+			if key == 'Include':
+				with open(value) as g:
+					preprocessed.write(g.read())
+				continue
+		preprocessed.write(line)
+
+	f.close()
+
 	parser = configparser.ConfigParser(
 			allow_no_value = True,
 			delimiters = ['='],
 			comment_prefixes = ['#'],
 			empty_lines_in_values = False)
-	parser.read(path)
+	parser.read_string(preprocessed.getvalue(), source=path)
+	# parser.write(sys.stderr)
 	return parser
 
 def make_parser(*args, **kwargs):
@@ -62,7 +82,33 @@ def make_parser(*args, **kwargs):
 			help = 'set an alternate log file')
 	return parser
 
-def init_with_config(options):
+def init_with_config(configpath):
+	"Reads configuration from given path and apply it to libalpm"
+	config = read_config(configpath)
+	arch = os.uname()[-1]
+	try:
+		config_options = config["options"]
+	except KeyError:
+		config_options = {}
+
+	pyalpm.options.root = config_options.get("rootdir", "/")
+	pyalpm.options.dbpath = config_options.get("dbpath", "/var/lib/pacman")
+	pyalpm.options.arch = config_options.get("architecture", arch)
+	pyalpm.options.logfile = config_options.get("logfile", "/var/log/pacman.log")
+	# set sync databases
+	for repo in config.sections():
+		if repo == "options":
+			continue
+		db = pyalpm.register_syncdb(repo)
+		# set server URL
+		rawurl = config[repo].get("Server", None)
+		if rawurl is not None:
+			url = rawurl.replace("$repo", repo)
+			url = url.replace("$arch", pyalpm.options.arch)
+		db.url = url
+
+def init_with_config_and_options(options):
+	"Reads configuration from file and commandline options, and apply it to libalpm"
 	pyalpm.initialize()
 
 	# read config file
@@ -91,7 +137,8 @@ def init_with_config(options):
 	if options.arch is not None:
 		pyalpm.options.arch = options.arch
 	else:
-		pyalpm.options.arch = config_options.get("architecture", "auto")
+		arch = os.uname()[-1]
+		pyalpm.options.arch = config_options.get("architecture", arch)
 
 	if options.logfile is not None:
 		pyalpm.options.logfile = options.logfile
@@ -102,6 +149,13 @@ def init_with_config(options):
 	for repo in config.sections():
 		if repo == "options":
 			continue
-		pyalpm.register_syncdb(repo)
+		db = pyalpm.register_syncdb(repo)
+		# set server URL
+		rawurl = config[repo].get("Server", None)
+		if rawurl is not None:
+			url = rawurl.replace("$repo", repo)
+			url = url.replace("$arch", pyalpm.options.arch)
+		db.url = url
+		# print("found DB", repo, url)
 
 # vim: set ts=4 sw=4 tw=0 noet:
